﻿using System;
using System.IO;
using System.IO.Compression;
using System.Text;
using System.Xml;

namespace CV8
{
    class Program
    {
        static void Main(string[] args)
        {
            // Xml Read/Write part
            XmlWriterWrite();
            XmlDocumentRead();
            XmlDocumentWrite();
            XmlReaderRead();

            // Xml Serialization Part
            MyXmlClass myXml = new MyXmlClass();
            myXml.X1 = 42;
            myXml.X2 = Math.E;
            myXml.Y = "Hello";
            myXml.Z = "World";
            MyXmlClass.SerializeToFile(myXml, "xmlSerializer.xml");
            // Try to change something before deserialization
            myXml = MyXmlClass.DeserializeFromFile("xmlSerializer.xml");
            Console.WriteLine(myXml);

            // Compression part
            using (FileStream output = File.OpenWrite("zippedXml.xml.gz"))
            using (FileStream input = File.OpenRead("xmlWriterAndXmlDocument.xml"))
            using (GZipStream zip = new GZipStream(output, CompressionMode.Compress))
            {
                input.CopyTo(zip);
            }
        }

        static void XmlWriterWrite()
        {
            using (XmlWriter writer = XmlWriter.Create("xmlWriterAndXmlDocument.xml"))
            {
                writer.WriteStartDocument();
                writer.WriteStartElement("breakfast_menu");

                writer.WriteStartElement("food");
                writer.WriteStartElement("name");
                writer.WriteString("Belgian Waffles");
                writer.WriteEndElement();
                writer.WriteStartElement("price");
                writer.WriteString("$5.95");
                writer.WriteEndElement();
                writer.WriteStartElement("description");
                writer.WriteString("Two of our famous Belgian Waffles with plenty of real maple syrup");
                writer.WriteEndElement();
                writer.WriteStartElement("calories");
                writer.WriteString("650");
                writer.WriteEndElement();
                writer.WriteEndElement();

                writer.WriteStartElement("food");
                writer.WriteStartElement("name");
                writer.WriteString("Strawberry Belgian Waffles");
                writer.WriteEndElement();
                writer.WriteStartElement("price");
                writer.WriteString("$7.95");
                writer.WriteEndElement();
                writer.WriteStartElement("description");
                writer.WriteString("Light Belgian waffles covered with strawberries and whipped cream");
                writer.WriteEndElement();
                writer.WriteStartElement("calories");
                writer.WriteString("900");
                writer.WriteEndElement();
                writer.WriteEndElement();

                writer.WriteEndElement();
                writer.WriteEndDocument();
            }
        }

        static void XmlDocumentRead()
        {
            XmlDocument doc = new XmlDocument();
            doc.Load("xmlWriterAndXmlDocument.xml");
            XmlElement root = doc.DocumentElement;

            foreach (XmlNode food in root.ChildNodes)
            {
                Console.WriteLine("Food:");

                foreach (XmlNode detail in food.ChildNodes)
                {
                    Console.WriteLine("\t{0}: {1}", detail.Name, detail.InnerText);
                }
            }
        }

        static void XmlDocumentWrite()
        {
            XmlDocument doc = new XmlDocument();
            doc.AppendChild(doc.CreateXmlDeclaration("1.0", Encoding.UTF8.HeaderName, null));
            XmlNode root = doc.AppendChild(doc.CreateElement("breakfast_menu"));
            XmlNode food;
            XmlNode actual;

            food = root.AppendChild(doc.CreateElement("food"));
            actual = doc.CreateElement("name");
            actual.InnerText = "Belgian Waffles";
            food.AppendChild(actual);
            actual = doc.CreateElement("price");
            actual.InnerText = "$5.95";
            food.AppendChild(actual);
            actual = doc.CreateElement("description");
            actual.InnerText = "Two of our famous Belgian Waffles with plenty of real maple syrup";
            food.AppendChild(actual);
            actual = doc.CreateElement("calories");
            actual.InnerText = "650";
            food.AppendChild(actual);

            food = root.AppendChild(doc.CreateElement("food"));
            actual = doc.CreateElement("name");
            actual.InnerText = "Strawberry Belgian Waffles";
            food.AppendChild(actual);
            actual = doc.CreateElement("price");
            actual.InnerText = "$7.95";
            food.AppendChild(actual);
            actual = doc.CreateElement("description");
            actual.InnerText = "Light Belgian waffles covered with strawberries and whipped cream";
            food.AppendChild(actual);
            actual = doc.CreateElement("calories");
            actual.InnerText = "900";
            food.AppendChild(actual);

            doc.Save("xmlDocumentAndXmlReader.xml");
        }

        static void XmlReaderRead()
        {
            using (XmlReader reader = XmlReader.Create("xmlDocumentAndXmlReader.xml"))
            {
                reader.ReadStartElement();

                while (reader.Read())
                {
                    if (reader.NodeType == XmlNodeType.Element)
                    {
                        if (reader.Name == "food")
                            Console.WriteLine("Food:");
                        else
                            Console.WriteLine("\t{0}: {1}", reader.Name, reader.ReadInnerXml());
                    }
                }
            }
        }
    }
}
